#!/bin/bash -ex
# topo B-A-C creates the following network topology:
#
#
#    netns:B          netns:A         netns:C
#
#       B  --- br0 ---  A  --- br1 ---  C
#        b1  B1   A1  a1 a2  A2   C1  c1
#
#
# should be run under unshare -mrun .
# use xnsenter {A,B,C} to enter into namespaces of A/B/C.
#
# Based on https://github.com/troglobit/smcroute/tree/master/test

rm -rf A B C
mkdir A B C
PID=$$

# `xunshare <dir> ...`  -- unshare net/mount/... namespaces to be referenced by dir
xunshare() {
    ref=$1
    shift
    touch $ref/mnt $ref/net
    unshare --mount=$ref/mnt --net=$ref/net "$@"
}

# `xnsenter <dir> ...`  -- nsenter net/mount/... namespaces to be referenced by dir
xnsenter() {
    ref=$1
    shift
    nsenter --mount=$ref/mnt --net=$ref/net --wd=. "$@"
}


# B-br0-A
xunshare B -- ip link set lo up
xnsenter B -- ip link add b1 type veth peer B1
xnsenter B -- ip link set B1 netns $PID
xnsenter B -- ip link set b1 up
ip link set B1 up

xunshare A -- ip link set lo up
xnsenter A -- ip link add a1 type veth peer A1
xnsenter A -- ip link set A1 netns $PID
xnsenter A -- ip link set a1 up
ip link set A1 up

ip link add br0 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set B1 master br0
ip link set A1 master br0
ip link set br0 up

# A-br1-C
xnsenter A -- ip link add a2 type veth peer A2
xnsenter A -- ip link set A2 netns $PID
xnsenter A -- ip link set a2 up
ip link set A2 up

xunshare C -- ip link set lo up
xnsenter C -- ip link add c1 type veth peer C1
xnsenter C -- ip link set C1 netns $PID
xnsenter C -- ip link set c1 up
ip link set C1 up

ip link add br1 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set C1 master br1
ip link set A2 master br1
ip link set br1 up


# addresses
xnsenter B -- ip addr add 10.0.0.1/24 dev b1
xnsenter A -- ip addr add 10.0.0.2/24 dev a1
xnsenter C -- ip addr add 20.0.0.1/24 dev c1
xnsenter A -- ip addr add 20.0.0.2/24 dev a2

# multicast routing
xnsenter B -- ip route add 224.0.0.0/4 dev b1
xnsenter C -- ip route add 224.0.0.0/4 dev c1

xnsenter A -- ip route add    224.0.0.0/4 dev a1   # NOTE on A 224.0.0.0/4 is routed
xnsenter A -- ip route append 224.0.0.0/4 dev a2   # to _both_ a1 and a2


# mount private /var/run so that smcrouted can be started
xnsenter A -- mount -t tmpfs none /var/run
xnsenter B -- mount -t tmpfs none /var/run
xnsenter C -- mount -t tmpfs none /var/run

echo
echo
echo
xnsenter B -- pwd


# open terminals in A B C for play
xnsenter B -- xfce4-terminal --title=B &
xnsenter C -- xfce4-terminal --title=C &
xnsenter A -- xfce4-terminal --title=A &
xnsenter A -- xfce4-terminal --title=A2 &
